/* Modified by takaharu 24/01/2004 */


/*
    Copyright (C) 2002 Terratron Technologies Inc.  All Rights Reserved.
    Author: Steven Osman, Morten Mikkelsen, Lionel Lemarie

    This file is part of sps2.

    Before using this file you MUST agree to the license agreement in
    the file LICENSE provided with this package.

    For more information visit:
    http://www.playstation2-linux.com/projects/sps2/
    http://window.terratron.com/~sosman/ps2linux/
    http://www.terratron.com/

    7/12/2003 - Sauce
    Initial version
*/
#include <stdio.h>

#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>

extern int midi_in_open();
extern int midi_in_close();
extern int midi_read();

int iFrame=0;
int iFrameS=0;
int iFrameH=0;
typedef int vertex_t[2];
vertex_t aVertices[]={
  {-150, -150}, {-150,  -50}, { -50, -100}, // 1st triangle
  { 150, -150}, { 150,  -50}, {  50, -100}, // 2nd triangle
  {  50,  150}, {  50,   50}, { -50,  100}, // 3rd triangle
};

int iVertexCount=sizeof(aVertices)/sizeof(vertex_t);

/**
 * This function will take the iVertexCount vertices defined in the aVertices
 * array and prepare them, along with an appropriate GIFTag, to be displayed.
 * pvBase points to a buffer for the data
 * returns the number of qwords put into pvBase
 */
int loadTriangleData(void *pvBase) {
  sps2GIFTag_t *pGIFTag=(sps2GIFTag_t *) pvBase;
  sps2GIFPackedRegister_t *pRegister=
    (sps2GIFPackedRegister_t *) ((int) pvBase+sizeof(sps2GIFTag_t));
  int iVertex, iScale;

  // Prepare the GIF tag  
  pGIFTag->i128=0;       // Blank it out
  pGIFTag->s.NLOOP=0;    // 0 entries.  We'll increment as we add entries
  pGIFTag->s.PRE=1;      // We are providing a valid PRIM value
  pGIFTag->s.EOP=1;      // End of packet -- draw me please!
  pGIFTag->s.PRIM=11;    // 3 = triangle | 8 = gourard shading
  pGIFTag->s.FLG=GIF_FLG_PACKED; // We're using the packed format for data
  pGIFTag->s.NREG=0;     // Start off with zero registers, increment as we
                         // populate the descriptions
  
  // Add a register.  It will be an RGBAQ register
  SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_RGBAQ);
  // Add another register.  It will by an XYZ2 register
  SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_XYZ2);
  
  // Loop through all vertices
  for (iVertex=0;iVertex<iVertexCount;iVertex++) {
    pGIFTag->s.NLOOP++;  // Increment the entry count in the tag
    
    pRegister->i128=0;   // Blank out this value
    pRegister->RGBAQ.s.A=0x7f; // Alpha = 127
    // Pick R, G, and B values just based on the vertex number and frame
//    pRegister->RGBAQ.s.R=(iVertex % 3==0 ? iFrame & 0x7f : 0);
/*
    pRegister->RGBAQ.s.R=(iVertex % 3==0 ? (0xff -iFrame*3) : 0);
    pRegister->RGBAQ.s.G=(iVertex % 3==1 ? (0xff-iFrameS*3) : 0);
    pRegister->RGBAQ.s.B=(iVertex % 3==2 ? (0xff-iFrameH*6) : 0);
*/
    pRegister->RGBAQ.s.R=0;
    pRegister->RGBAQ.s.G=0;
    pRegister->RGBAQ.s.B=0;
    switch(iVertex %3){
	case(0):
	    if(iFrame<16) pRegister->RGBAQ.s.R=0xff -iFrame*iFrame;
	    break;
	case(1):
	    if(iFrameS<16) pRegister->RGBAQ.s.G=0xff-iFrameS*iFrameS;
	    break;
	case(2):
	    if(iFrameH<4) pRegister->RGBAQ.s.B=0xff-iFrameH*iFrameH*4;
	    break;
    }
    
    // Increment register data pointer
    pRegister++;
    
    pRegister->i128=0;  // Blank out this value
  //  iScale=1+(iFrame & 31);  // We'll scale the points by half these amounts

    if (iVertex>5) {
	iScale=1+(iFrameS & 62);  // We'll scale the points by half these amounts
    }else{
	iScale=1+(iFrame & 62);
    }
    pRegister->XYZ2.s.X=(2048 << 4) + ((aVertices[iVertex][0]*iScale) >> 1);
    pRegister->XYZ2.s.Y=(2048 << 4) + ((aVertices[iVertex][1]*iScale) >> 1);
    pRegister->XYZ2.s.Z=128;
    
    // Increment register data pointer
    pRegister++;
  }

  // Compute the difference between our register data pointer and our initial
  // pointer.  This is the byte count of data we put in.  Shifted right four
  // times gives us the qword count
  return ((int) pRegister -(int) pvBase) >> 4;
}

/**
 * This is the main function of our tutorial.  It will perform some simple
 * initialization and then set up a small loop to repeatedly draw our
 * triangles.
 */
int main() {
  int iSPS2Descriptor;      // The handle we use for sps2
  sps2Memory_t *pMemory;    // Will store information about memory we allocate
  int iQWC;
  Dn_CHCR_t chcrValue;      // This is used to control the DMA transfer

  // chcr is used to control, start, stop DMA transfers.
  // Since we need the same chcr value each time we initialize this
  // particular dma transfer, we may as well prepare the value first.
  chcrValue.i32=0;                  // Blank everything out
  chcrValue.s.MOD=CHCR_MOD_NORMAL;  // We'll do a "normal" dma transfer
  chcrValue.s.STR=1;                // STR=1 = start transfer

  iSPS2Descriptor=sps2Init();       // Initialize sps2

  // We'll allocate 4096 bytes in chunks of 4K (i.e. one single chunk).
  // We want to use this memory uncached so that writes to the memory are
  // immediately committed.
  pMemory=sps2Allocate(4096, SPS2_MAP_BLOCK_4K | SPS2_MAP_UNCACHED,
		       iSPS2Descriptor);

  // Initialize the screen.  Zero tells sps2UScreenInit to exit gracefully
  // when it receives a signal.
  sps2UScreenInit(0);

  midi_in_open();

  while (++iFrame) {

    iFrameS++;
    iFrameH++;

    // Clear the new screen.  Let's make it a dark purple background.
    sps2UScreenClear(0x40, 0, 0x40);

    midi_read();

    // Load our polygon data.  This function returns the number of
    // qwords (QWC = qword count.  qword = 16 bytes)
    iQWC=loadTriangleData(pMemory->pvStart);

    // Tell DMA we'd like to transfer iQWC qwords
    *EE_D2_QWC=iQWC;

    // Get the physical address of our data and pass it to DMA
    *EE_D2_MADR=sps2GetPhysicalAddress(pMemory->pvStart, pMemory);

    // Start the DMA transfer
    *EE_D2_CHCR=chcrValue.i32;

    // Wait for the DMA transfer to finish
    sps2WaitForDMA(2, iSPS2Descriptor);

    // Swap displays now that we're done
    sps2UScreenSwap();
  }

  midi_in_close();
  
  // Shut down the screen
  sps2UScreenShutdown();

  // Free the memory
  sps2Free(pMemory);

  // and close sps2
  sps2Release(iSPS2Descriptor);
  return 0;
}
