
/*---------------------------------------------------------------------*/
/*      Takaharu  midi.c for ver. 003				*/
/*---------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

  /*
   * A test program for reading to /dev/midi.
   *
   */

#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include "midi.h"

int midi_branch(unsigned char data);
int midi_ch_message(unsigned char data);
int midi_data(unsigned char data);
int midi_key_off(unsigned char data);
int midi_key_on(unsigned char data);
int midi_clock();
int midi_real_time(unsigned char data);
int midi_ctrlAndmode(unsigned char data);
int midi_disp_poly_press();
int midi_poly_press(unsigned char data);
int midi_stop();
int midi_continue();
int midi_start();
int active_sens();
int midi_ch_mode_message();
int all_notes_off();
int all_sound_off();
int reset_all_controllers();
int local_control();
int omni_off();
int omni_on();
int poly_off();
int poly_on();
int error();
int midi_control_change();
int midi_disp_control_change();
int midi_program_change(unsigned char data);
int midi_disp_program();
int midi_ch_press(unsigned char data);
int midi_disp_ch_press();
int midi_bend_change(unsigned char data);
int midi_disp_bend();
int midi_song_position(unsigned char data);
int midi_disp_song_position();
int midi_exclusive(unsigned char data);
int midi_exclusive_data(unsigned char data);
int midi_error(unsigned char data);
  
int midi_disp_key_off();
int midi_disp_key_on();
int drum_disp();


extern int iFrame;
extern int iFrameH;
extern int iFrameS;

#define MIDI_DEV "/dev/midi00"
static int fd;

int midi_in_open(void)
{

    if ((fd = open(MIDI_DEV, O_RDONLY|O_NDELAY )) == -1) {
	printf("Unable to open MIDI device !\n");
	return(-1);
    }else
	 printf("MIDI device is open !\n");
    return(0);
}
int midi_read(void)
{
   char PrFlag;
   int dt;
   unsigned char buff[64];
   char str0[64];

   PrFlag=0;
   while ( (dt=read(fd,buff,1)) == 1 ) { 
      if (buff[0]==0xf8) continue;
        PrFlag=1;
	str0[0]=(buff[0] >>4 );
        if (str0[0]>9) str0[0]+='a'-10;
        else str0[0]+='0';
	str0[1]=(buff[0] & 0x0f );
        if (str0[1]>9) str0[1]+='a'-10;
        else str0[1]+='0';
	str0[2]='\0';
  //printf("MIDI: %s ",str0);
     midi_branch(buff[0]);
   }
   //if (PrFlag==1)  printf("\n");
   return(dt);
}

int midi_in_close(void)
{
    close(fd);
    printf("MIDI device is closed !\n");
    return(0);

}
//--------
unsigned char midi_status,midi_ch,midi_data_length;
unsigned char key_number,key_velocity,key_pressure;
unsigned char midi_data_1st,midi_data_2nd; 
unsigned char midi_program_map[16];
unsigned char midi_ch_press_map[16];

int midi_branch(unsigned char data)
{
	if (data < 0x80) {
	        midi_data(data);
		return(0);
	} else if (data > 0xf7) {
		midi_real_time(data);
		return(0);
	} else if (data >= 0xf0 ) {
		midi_status = 0xf0;
	 	midi_exclusive(data);
		return(0);
	} else midi_ch_message(data); 
    return(0);
}
int midi_ch_message(unsigned char data)
{
	midi_status = data & 0xf0;
	midi_ch = data & 0x0f;
	if (midi_status == 0xc0) midi_data_length=1;
	else if (midi_status == 0xd0) midi_data_length=1;
	else midi_data_length=2;
	return(0);
}
int midi_data(unsigned char data)
{
	switch(midi_status) {
		case 0x80:
			midi_key_off(data);
			break;
		case 0x90:
			midi_key_on(data);
			break;
		case 0xa0:
			midi_poly_press(data);
			break;
		case 0xb0:
			midi_ctrlAndmode(data);	
			break;
		case 0xc0:
			midi_program_change(data);
			break;
		case 0xd0:
			midi_ch_press(data);
			break;
		case 0xe0:
			midi_bend_change(data);
			break;
		case 0xf0:
			midi_exclusive_data(data);
			break;
		case 0xf2:
			midi_song_position(data);
			break;
		default:
			midi_error(data);
	}
	return(0);
}
int midi_key_off(unsigned char data)
{
	if (midi_data_length==2) {
		key_number=data;
		midi_data_length--;
	} else {
		key_velocity=data;
		midi_data_length=2;
		midi_disp_key_off();
	}
	return(0);
}
int midi_disp_key_off()
{
	printf("%d CH Note %d Key Off\n",midi_ch,key_number);
	return(0);
}
int midi_key_on(unsigned char data)
{
	if (midi_data_length==2) {
		key_number=data;
		midi_data_length--;
	} else {
		key_velocity=data;
		midi_data_length=2;
		if (key_velocity==0) midi_disp_key_off();
		else midi_disp_key_on();
	}
	return(0);
}
int midi_disp_key_on()
{
	printf("%d CH Note %d Key On Velo %d\n",midi_ch,key_number,key_velocity);

	if (midi_ch==9) drum_disp();

	return(0);
}

int drum_disp()
{
	char drum_grp;

	drum_grp=drum_map[key_number];
//koreya
	if( drum_grp==1) iFrame=0;
	if( drum_grp==2) iFrameS=0;
	if( drum_grp==3) iFrameH=0;

	return(0);
}

int midi_poly_press(unsigned char data)
{
	if (midi_data_length==2) {
		key_number=data;
		midi_data_length--;
	} else {
		key_pressure=data;
		midi_data_length=2;
		midi_disp_poly_press();
	}
	return(0);
	
}
int midi_disp_poly_press()
{
	printf("%d CH Note %d Poly Press %d\n",midi_ch,key_number,key_pressure);
	return(0);
}

int midi_ctrlAndmode(unsigned char data)
{
	if (midi_data_length==2) {
		midi_data_1st=data;
		midi_data_length--;
	} else {
		midi_data_2nd=data;
		midi_data_length=2;
		if (midi_data_1st > 0x77) midi_ch_mode_message();
		else midi_control_change();
	}
	return(0);
	
}
int midi_real_time(unsigned char data)
{
	switch(data) {
		case 0xf8:
			midi_clock();
			break;
		case 0xf9:
			break;
		case 0xfa:
			midi_start();
			break;
		case 0xfb:
			midi_continue();
			break;
		case 0xfc:
			midi_stop();
			break;
		case 0xfd:
			break;
		case 0xfe:
			active_sens();
			break;
		case 0xff:
			break;
	}
	return(0);
}
int midi_clock()
{
	printf("F8 ");
	return(0);
}
int midi_start()
{
	printf("START \n");
	return(0);
}
int midi_continue()
{
	printf("CONT \n");
	return(0);
}
int midi_stop()
{
	printf("STOP \n");
	return(0);
}
int active_sens()
{
	printf("FE ");
	return(0);
}
int midi_ch_mode_message()
{
	switch(midi_data_1st) {
		case 0x7b:
			all_notes_off();
			break;
		case 0x78:
			all_sound_off();
			break;
		case 0x79:
			reset_all_controllers();
			break;
		case 0x7a:
			local_control();
			break;
		case 0x7c:
			omni_off();
			break;
		case 0x7d:
			omni_on();
			break;
		case 0x7e:
			poly_off();
			break;
		case 0x7f:
			poly_on();
			break;
		default:
			error();
			break;
	}
	return(0);
}
int all_notes_off()
{
	printf("%d CH all_notes_off\n",midi_ch);
	return(0);
}
int all_sound_off()
{
	printf("%d CH all_sound_off\n",midi_ch);
	return(0);
}
int reset_all_controllers()
{
	printf("%d CH reset_all_controllers\n",midi_ch);
	return(0);
}
int local_control()
{
	printf("%d CH local_control\n",midi_ch);
	return(0);
}
int omni_off()
{
	printf("%d CH omni_off\n",midi_ch);
	return(0);
}
int omni_on()
{
	printf("%d CH omni_on\n",midi_ch);
	return(0);
}
int poly_off()
{
	printf("%d CH poly_off\n",midi_ch);
	return(0);
}
int poly_on()
{
	printf("%d CH poly_on\n",midi_ch);
	return(0);
}
int error()
{
	printf("%d CH mode message error.\n",midi_ch);
	return(0);
}
int midi_control_change()
{
/*
	unsigned char *ctrl_data_pointer;

	ctrl_data_pointer= &midi_ctrl_data_map;	
	ctrl_data_pointer += ch*120+midi_data_1st;
	*ctrl_data_pointer = midi_data_2nd;

	switch(midi_data_1st) {
		case 0x64
*/
	midi_disp_control_change();
	return(0);
}
int midi_disp_control_change()
{
	printf("%d CH Control %d Data %d \n",midi_ch,midi_data_1st,midi_data_2nd);
	return(0);
}

int midi_program_change(unsigned char data)
{
	midi_program_map[midi_ch]=data;
	midi_disp_program();
        midi_data_1st=data;
	return(0);
}
int midi_disp_program()
{
	printf("%d CH Program %d \n",midi_ch,midi_data_1st);
	return(0);
}
int midi_ch_press(unsigned char data)
{
	midi_ch_press_map[midi_ch]=data;
	midi_disp_ch_press();
	return(0);
}
int midi_disp_ch_press()
{
	printf("%d CH Press %d \n",midi_ch,midi_data_1st);
	return(0);
}

int midi_bend_change(unsigned char data)
{
	if (midi_data_length==2) {
		midi_data_1st=data;
		midi_data_length--;
	} else {
		midi_data_2nd=data;
		midi_data_length=2;
		midi_disp_bend();
	}
	return(0);
}
int midi_disp_bend()
{
	printf("%d CH Bend %x %x \n",midi_ch,midi_data_1st,midi_data_2nd);
	return(0);
}
int midi_song_position(unsigned char data)
{
	if (midi_data_length==2) {
		midi_data_1st=data;
		midi_data_length--;
	} else {
		midi_data_2nd=data;
		midi_data_length=2;
		midi_disp_song_position();
	}
	return(0);
}
int midi_disp_song_position()
{
	printf("Song Position %x %x \n",midi_data_1st,midi_data_2nd);
	return(0);
}
int midi_exclusive(unsigned char data)
{
	switch(data){
		case 0xf2:
			midi_status=0xf2;
			midi_data_length=2;
			break;
		case 0xf0:
//			printf("System Exclusive  Start\n");
			printf("Sys.Ex. ");
			break;
		case 0xf7:
//  			printf(" System Exclusive End \n");
  			printf(" \n");
			break;
		default:
			break;
	}
 	return(0);
}
int midi_exclusive_data(unsigned char data)
{
//	printf("System Exclusive data %2x\n",data);
	printf("%x ",data);
	return(0);
}
int midi_error(unsigned char data)
{
	printf("Error \n");
	return(-1);
}

